package cn.fastfun.controller.api;

import cn.fastfun.controller.dto.asset.BaseInfoDTO;
import cn.fastfun.controller.dto.asset.RentInfoDTO;
import cn.fastfun.controller.dto.asset.RentInfoFigureDTO;
import cn.fastfun.controller.param.*;
import cn.fastfun.service.SysExcelFieldService;
import cn.fastfun.service.SysService;
import cn.fastfun.service.impl.AppAssetProfitServiceImpl;
import com.bridge.dto.*;
import com.bridge.exception.ApiRuntimeException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Bridge AutoGen
 */
@Slf4j
@Api(tags = {"2.0 资产收益分析"})
@RestController
@RequestMapping("/api/v1/appassetprofit")
public class ApiAssetProfitController {

    @Autowired
    AppAssetProfitServiceImpl appAssetService;

    @Resource
    SysExcelFieldService sysExcelFieldService;

    @ApiOperation(value = "基础信息")
    @RequestMapping(value = "getBaseInfo", method = RequestMethod.POST)
    public ApiDTO getBaseInfo(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) AssetProfitParam param) throws Exception {


        BaseInfoDTO baseInfo = appAssetService.getBaseInfo(param);
        return ApiDTO.ok("成功", baseInfo);
    }

    @ApiOperation(value = "其他信息列表")
    @RequestMapping(value = "getProfitInfo", method = RequestMethod.POST)
    public ApiDTO getAssetProfitInfo(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) AssetProfitParam param) throws Exception {

        // 租赁信息
        Object response = null;
        if (param.getTableOrder() != null && param.getTableOrder().equals(1)) {
            param.setFigureOrNot(false);
            Object rentInfoDTO = appAssetService.getRentInfo(param);
            response = (RentInfoDTO) rentInfoDTO;
        }

        return ApiDTO.ok("成功", response);
    }

    @ApiOperation(value = "其他信息绘图")
    @RequestMapping(value = "getProfitInfoFigure", method = RequestMethod.POST)
    public ApiDTO getAssetProfitInfoFigure(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) AssetProfitParam param) throws Exception {

        // 租赁信息
        Object response = null;
        if (param.getTableOrder() != null && param.getTableOrder().equals(1)) {
            param.setFigureOrNot(true);
            param.setIndex(0);
            param.setLength(0);
            Object rentInfoFigureDTO = appAssetService.getRentInfo(param);
            response = (RentInfoFigureDTO) rentInfoFigureDTO;
        }

        return ApiDTO.ok("成功", response);
    }

    @ApiOperation(value = "导出")
    @RequestMapping(value = "export", method = RequestMethod.POST)
    public void export(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) AssetProfitParam param, HttpServletResponse response) {

        try {
            // 租赁信息

            if (param.getTableOrder() != null && param.getTableOrder().equals(1)) {
                param.setFigureOrNot(false);
                RentInfoDTO rentInfoDTO = (RentInfoDTO) appAssetService.getRentInfo(param);
                List<Map<String, Object>> data = new ArrayList<>();
                rentInfoDTO.getAllInfoList().forEach(p -> {
                    data.add(new HashMap<String, Object>() {{
                        put("time", p.getTime());
                        put("type", p.getType());
                        put("amount", p.getAmount());
                        put("orderNo", p.getOrder());
                        put("remark", p.getDescription());
                    }});
                });
                sysExcelFieldService.outExcelFile(sysExcelFieldService.export("rent_info", data), response, "租赁信息.xlsx");
            }
        } catch (Exception e) {
            throw new ApiRuntimeException("模板下载失败");

//        return ApiDTO.ok("成功");
        }
    }
}